@@ -1008,7 +1008,7 @@ def test_markers_from_parametrize(testdir):
 def test_pytest_param_id_requires_string():
     with pytest.raises(TypeError) as excinfo:
         pytest.param(id=True)
-    msg, = excinfo.value.args
+    (msg,) = excinfo.value.args
     if six.PY2:
         assert msg == "Expected id to be a string, got <type 'bool'>: True"
     else:
@@ -1025,7 +1025,7 @@ def test_pytest_param_warning_on_unknown_kwargs():
         # typo, should be marks=
         pytest.param(1, 2, mark=pytest.mark.xfail())
     assert warninfo[0].filename == __file__
-    msg, = warninfo[0].message.args
+    (msg,) = warninfo[0].message.args
     assert msg == (
         "pytest.param() got unexpected keyword arguments: ['mark'].\n"
         "This will be an error in future versions."
