@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import subprocess
 import sys
 
@@ -6,6 +7,8 @@ import py
 import _pytest
 import pytest
 
+pytestmark = pytest.mark.slow
+
 MODSET = [
     x
     for x in py.path.local(_pytest.__file__).dirpath().visit("*.py")
